// Copyright 2024 Esri
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import SwiftUI
import ArcGIS


struct ContentView: View {

    @StateObject private var model = Model()

    @State private var geoViewExtent: Envelope?

    @State private var selectedWhereClause: WhereClause = .government

    var body: some View {

        MapView(map: model.map)

            .onVisibleAreaChanged { newVisibleArea in
                geoViewExtent = newVisibleArea.extent
            }

            .toolbar {
                ToolbarItemGroup(placement: .bottomBar) {

                    Spacer()
                    Picker("Choose an SQL `where` clause", selection: $selectedWhereClause) {
                        ForEach(WhereClause.allCases, id: \.self) { clause in
                            Text(clause.expression)
                        }

                        .task(id: selectedWhereClause) {
                            guard let geoViewExtent = geoViewExtent else { return }
                            await model.queryFeatureLayer(with: selectedWhereClause, in: geoViewExtent)
                        }

                    }
                    Spacer()

                }
            }

    }

}

private extension ContentView {

    enum WhereClause: CaseIterable, Equatable {
        case government, residential, farm, taxRate10853, taxRate10860, landValueGreaterThan, landValueLessThan

        var expression: String {
            switch self {
            case .government: return "UseType = 'Government'"
            case .residential: return "UseType = 'Residential'"
            case .farm: return "UseType = 'Irrigated Farm'"
            case .taxRate10853: return "TaxRateArea = 10853"
            case .taxRate10860: return "TaxRateArea = 10860"
            case .landValueGreaterThan: return "Roll_LandValue > 1000000"
            case .landValueLessThan: return "Roll_LandValue < 1000000"
            }
        }
    }

    class Model: ObservableObject {

        let map: Map = {
            let map = Map(basemapStyle: .arcGISTopographic)

            map.initialViewpoint = Viewpoint(latitude: 34.03000, longitude: -118.80500, scale: 6e4)

            let parcelsURL = URL(string: "https://services3.arcgis.com/GVgbJbqm8hXASVYi/arcgis/rest/services/LA_County_Parcels/FeatureServer/0")!
            let featureTable = ServiceFeatureTable(url: parcelsURL)
            let featureLayer = FeatureLayer(featureTable: featureTable)
            map.addOperationalLayer(featureLayer)

            return map
        }()


        func queryFeatureLayer(with selectedWhereClause: WhereClause, in extent: Envelope) async {
            guard
                let parcelsFeatureLayer = map.operationalLayers.first as? FeatureLayer,
                let featureTable = parcelsFeatureLayer.featureTable
            else { return }
            parcelsFeatureLayer.clearSelection()

            let parameters = QueryParameters()
            parameters.whereClause = selectedWhereClause.expression
            parameters.geometry = extent

            do {
                let result = try await featureTable.queryFeatures(using: parameters)
                let features = result.features()
                parcelsFeatureLayer.selectFeatures(features)
            } catch {
                print(error)
            }
        }

    }

}


